###############################################################################
## Copyright  1999 Klaus Alexander Seistrup @ Magnetic Ink, Copenhagen, DK
##
##   Author: 1999 Klaus Alexander Seistrup
##  Created: Friday, 3rd December 1999
##  Version: 0.1
## Homepage: http://www.magnetic-ink.dk/download/snippets/
##
## @(#) $Id: vcs.py,v 1.5 1999/12/04 19:42:04 kas Exp $
##
## Freshmeat appindex record no. 944337461.
##
## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by the Free
## Software Foundation; either version 2 of the License, or (at your option)
## any later version.
##
## This program is distributed in the hope that it will be useful, but with-
## out any warranty; without even the implied warranty of merchantability or
## fitness for a particular purpose.  See the GNU General Public License for
## more details.
##
## You should have received a copy of the GNU General Public License along
## with this program; if not, please write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307, USA.
###############################################################################
"""
NAME
	vcs - use virtual console memory to save/restore console

SYNOPSIS
	import vcs

	vcs.pushtty ()
	vcs.poptty ()

	vcs.help()
	vcs.version()

	v = vcs.VCS()
	v.push()
	v.pop()

DESCRIPTION
	This module requires the vcsa(4) virtual console memory
	character device.

	Use the pushtty() function to save the contents of the tty
	associated with standard input.  If standard input is not
	attached to a tty, /dev/null is used silently and the call
	will have no visual effect.

	Running this under X, i.e. using /dev/ttyp?, will raise an
	IOError.

	Console contents are restored by a call to poptty().  The
	calls nest -- there must be exactly one poptty() call for
	each pushtty() call.

RETURN VALUE
	None

BUGS
	Linux only.

SEE ALSO
	vcs(4), mc(1)
"""

# The VCS class
class VCS:
	"""VCS - class to manipulate the vcs(4) character device"""
	def __init__ (self, fp=None):
		import os, sys, string
		if not fp:
			fp = sys.stdin
		# end if
		if fp.isatty ():
			self.ttyname = os.ttyname (fp.fileno ())
		else:
			self.ttyname = "/dev/null"
		# end if
		##########################################################
		# This is a horrible hack: we assume that /dev/ttyN
		# corresponds to /dev/vcsaN.  It will, of course,
		# fail for e.g. /dev/ttypN.  Any suggestions?
		self.vcsname = string.replace (self.ttyname, "tty", "vcsa")
		self.__stack = []
	# end def __init__
	def push (self):
		""".push() - push tty onto stack, thus saving console"""
		fp = open (self.vcsname, "r")
		self.__stack.append (fp.read ())
		fp.close ()
	# end def push
	def pop (self):
		""".pop() - pop tty from stack, thus restoring console"""
		if self.__stack:
 			fp = open (self.vcsname, "w")
			fp.write (self.__stack.pop ())
			fp.close ()
		else:
			raise IndexError, "nothing to pop"
		# end if
	# end def pop
# end class vcs

# The global VCS object, please do not touch
__TheVCS__ = None

# Initialize VCS object
def console_init ():
	global __TheVCS__
	if not __TheVCS__:
		__TheVCS__ = VCS ()
	# end if
	return __TheVCS__
# end def console_init

# Save console
def console_save ():
	"""pushtty() - push tty onto stack"""
	vcs = console_init ()
	vcs.push ()
# end def console_save

# Restore console
def console_restore ():
	"""poptty() - pop tty from stack"""
	vcs = console_init ()
	vcs.pop ()
# end def console_restore

# Shorthand
pushtty	= console_save
poptty	= console_restore

__version__	= "@(#) $Id: vcs.py,v 1.5 1999/12/04 19:42:04 kas Exp $\0"

# Display version information
def version ():
	"""version - display version information"""
	from string import split
	rcs = split (__version__)
	print "vcs 0.%s (%s)" % tuple (rcs[3:5])
# end def version

# Display __doc__ (BOF)
def help ():
	"""help - print __doc__ associated with module"""
	print __doc__
# end def help

#
## Soli Deo Gloria ############################################################
